import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["children", "defaultValue", "disabled", "inputMode", "isOneTimeCode", "isPassword", "length", "mask", "onChange", "placeholder", "size", "status", "value"];
import PropTypes from 'prop-types';
import { forwardRef, useCallback, useEffect, useRef } from 'react';
import useMergeRefs from '@s-ui/react-hooks/lib/useMergeRefs';
import { PinInputContextProvider } from './PinInputContext';
import { BASE_CLASSNAME, MASK, SIZES } from './config.js';
import { actions as pinInputActions, usePinInputReducer } from './reducer';
import useKeyPress from './hooks/useKeyPress';
import useUpdateEffect from './hooks/useUpdateEffect';
import PinInputChildren from './PinInputChildren';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
var CLASSNAME = BASE_CLASSNAME;
var PinInput = /*#__PURE__*/forwardRef(function (_ref, forwardedRef) {
  var children = _ref.children,
      _ref$defaultValue = _ref.defaultValue,
      defaultValue = _ref$defaultValue === void 0 ? '' : _ref$defaultValue,
      disabled = _ref.disabled,
      inputMode = _ref.inputMode,
      _ref$isOneTimeCode = _ref.isOneTimeCode,
      isOneTimeCode = _ref$isOneTimeCode === void 0 ? true : _ref$isOneTimeCode,
      _ref$isPassword = _ref.isPassword,
      isPassword = _ref$isPassword === void 0 ? false : _ref$isPassword,
      _ref$length = _ref.length,
      length = _ref$length === void 0 ? 6 : _ref$length,
      mask = _ref.mask,
      onChange = _ref.onChange,
      _ref$placeholder = _ref.placeholder,
      placeholder = _ref$placeholder === void 0 ? '' : _ref$placeholder,
      _ref$size = _ref.size,
      size = _ref$size === void 0 ? SIZES.MEDIUM : _ref$size,
      status = _ref.status,
      value = _ref.value,
      props = _objectWithoutPropertiesLoose(_ref, _excluded);

  var innerRef = useRef();
  var targetRef = useRef();

  var _usePinInputReducer = usePinInputReducer({
    mask: mask,
    defaultValue: defaultValue,
    value: value,
    disabled: disabled
  }),
      reducerStore = _usePinInputReducer[0],
      dispatch = _usePinInputReducer[1];

  var innerValue = reducerStore.innerValue,
      focusPosition = reducerStore.focusPosition,
      elements = reducerStore.elements;
  useUpdateEffect(function () {
    var innerValue = typeof value === 'string' ? value.split('') : value || [];
    dispatch(pinInputActions.setValue({
      innerValue: innerValue
    }));
  }, ["" + value, dispatch]);
  useEffect(function () {
    dispatch(pinInputActions.setDisabled({
      disabled: disabled
    }));
  }, [disabled, dispatch]);
  useEffect(function () {
    dispatch(pinInputActions.setMask({
      mask: mask
    }));
  }, [mask, dispatch]);
  var setFocus = useCallback(function (position) {
    dispatch(pinInputActions.setFocus({
      focusPosition: position
    }));
  }, [dispatch]);
  useKeyPress(function (event) {
    dispatch(pinInputActions.setKey({
      event: event,
      onChange: onChange
    }));
  }, {
    target: targetRef,
    onChange: onChange
  });
  var getIndex = useCallback(function (node) {
    return elements.indexOf(node);
  }, [elements]);
  useUpdateEffect(function () {
    setFocus(focusPosition);
  }, [focusPosition]);
  return /*#__PURE__*/_jsxs("div", {
    className: CLASSNAME,
    ref: targetRef,
    children: [/*#__PURE__*/_jsx(PinInputContextProvider, {
      disabled: reducerStore.disabled,
      dispatch: dispatch,
      getIndex: getIndex,
      inputMode: inputMode,
      isOneTimeCode: isOneTimeCode,
      isPassword: isPassword,
      mask: reducerStore.mask,
      placeholder: placeholder,
      setFocus: setFocus,
      size: size,
      status: status,
      targetRef: innerRef,
      value: innerValue,
      children: /*#__PURE__*/_jsx(PinInputChildren, {
        length: length,
        children: children
      })
    }), /*#__PURE__*/_jsx("input", {
      type: "hidden",
      value: innerValue.filter(Boolean).join(''),
      ref: useMergeRefs(innerRef, forwardedRef)
    })]
  });
});
PinInput.displayName = 'PinInput';
export default PinInput;