import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["isFullWidth"];
import { forwardRef, useEffect, useRef, useCallback, useMemo } from 'react';
import PropTypes from 'prop-types';
import cx from 'classnames';
import useMergeRefs from '@s-ui/react-hooks/lib/useMergeRefs';
import { BASE_CLASSNAME, MASK } from './config.js';
import { usePinInputContext } from './PinInputContext';
import { actions as pinInputActions } from './reducer';
import { jsx as _jsx } from "react/jsx-runtime";
var CLASSNAME = BASE_CLASSNAME + "Field";

var getClassName = function getClassName(_ref) {
  var _cx;

  var size = _ref.size,
      status = _ref.status,
      isFullWidth = _ref.isFullWidth;
  return cx(CLASSNAME, (_cx = {}, _cx[CLASSNAME + "--size-" + size] = size, _cx[CLASSNAME + "--status-" + status] = status, _cx[CLASSNAME + "--fullWidth"] = isFullWidth, _cx));
};

var getInputMode = function getInputMode(_ref2) {
  var _maskToInputMode;

  var inputMode = _ref2.inputMode,
      mask = _ref2.mask;
  var maskToInputMode = (_maskToInputMode = {}, _maskToInputMode[MASK.NUMBER] = 'numeric', _maskToInputMode[MASK.ALPHABETIC] = 'text', _maskToInputMode[MASK.ALPHANUMERIC] = 'text', _maskToInputMode);
  return inputMode || maskToInputMode[mask];
};

var PinInputField = /*#__PURE__*/forwardRef(function (_ref3, forwardedRef) {
  var isFullWidth = _ref3.isFullWidth,
      props = _objectWithoutPropertiesLoose(_ref3, _excluded);

  var innerRef = useRef();

  var _usePinInputContext = usePinInputContext(),
      disabled = _usePinInputContext.disabled,
      dispatch = _usePinInputContext.dispatch,
      getIndex = _usePinInputContext.getIndex,
      inputMode = _usePinInputContext.inputMode,
      isOneTimeCode = _usePinInputContext.isOneTimeCode,
      isPassword = _usePinInputContext.isPassword,
      mask = _usePinInputContext.mask,
      _usePinInputContext$o = _usePinInputContext.onChange,
      onChange = _usePinInputContext$o === void 0 ? function () {
    return null;
  } : _usePinInputContext$o,
      placeholder = _usePinInputContext.placeholder,
      setFocus = _usePinInputContext.setFocus,
      size = _usePinInputContext.size,
      status = _usePinInputContext.status,
      _usePinInputContext$v = _usePinInputContext.value,
      value = _usePinInputContext$v === void 0 ? [] : _usePinInputContext$v;

  var index = getIndex(innerRef.current);
  var setElement = useCallback(function (node) {
    dispatch(pinInputActions.setElement({
      node: node
    }));
  }, [dispatch]);
  var removeElement = useCallback(function (node) {
    dispatch(pinInputActions.removeElement({
      node: node
    }));
  }, [dispatch]);
  useEffect(function () {
    if (innerRef.current) {
      setElement(innerRef.current);
    }

    return function () {
      return removeElement(innerRef.current);
    };
  }, [innerRef, setElement, removeElement]);

  var onFocusHandler = function onFocusHandler() {
    if (index >= 0) {
      setFocus(index);
    }
  };

  var inputModeMemoized = useMemo(function () {
    return getInputMode({
      inputMode: inputMode,
      mask: mask
    });
  }, [mask, inputMode]);
  return /*#__PURE__*/_jsx("input", _extends({
    className: getClassName({
      size: size,
      status: status,
      isFullWidth: isFullWidth
    }),
    disabled: disabled,
    maxLength: "1",
    onChange: onChange,
    onClick: onFocusHandler,
    onFocus: onFocusHandler,
    placeholder: placeholder,
    inputMode: inputModeMemoized,
    ref: useMergeRefs(innerRef, forwardedRef),
    value: value[index] || ''
  }, isPassword && {
    type: 'password'
  }, isOneTimeCode && {
    autoComplete: 'one-time-code'
  }, props));
});
PinInputField.displayName = 'PinInputField';
export default PinInputField;